; Author K.Sarnath., Dec 21st , 2000. This is a new version.
; This file will assmeble to 1024 bytes . So that this
; spans 2 sectors. Thus the boot floppy layout will be

; Sector 1 ->       Boot area. (bootsect.asm)
; Sector 2 and 3 -> 2nd stage. (2ndstage.asm)
; Sector 4 ->       Kernel
; This is intentional. coz, the kernel starts at a 3-sector boundary.
; So in bootsect.asm , i would load the kernel 3 sectors at a time..
; 18 sectors is perfectly divisble by 3. Hence faster !
; Rawrite author says that "some BIOSes donot support reads in large number".
; Hence i dropped the idea of loading 18 sectors at a time. :-(


DMASEGMENT      EQU 1000H     ; 64k allocated for it.
GDTSEGMENT      EQU 2000H     ; Abs address 20000h
KERNEL_PAGES    EQU 09h       ; Specify kernelpages here.

KPGD    EQU      3000H        ;CR3 = 30000h

PG_PRESENT      EQU     01H
PG_USER         EQU     04H
PG_WRITE        EQU     02H
PAGE_SHIFT      EQU     12
PD_START        EQU     0C00H
INTA01          EQU     021H
INTA02          EQU     0A1H

STRUC DESCRIPTOR
                limit_0_15      resw      0001h
                base_0_15       resw      0001h
                base_16_23      resb      01h
                attribute       resb      01h
                gran            resb      01h
                base_24_31      resb      01h
ENDSTRUC

[BITS 16]
SECTION .text
        cli
        jmp     Start

        ;Data Area
        ;---------
        i82591  resb      1
        i82592  resb      1
Gdt:
null:   ISTRUC  DESCRIPTOR
        at limit_0_15 , dw 0000h
        at base_0_15  , dw 0000h
        at base_16_23 , db 00h
        at attribute  , db 00h
        at gran       , db 00h
        at base_24_31 , db 00h
        IEND

data:   ISTRUC DESCRIPTOR
        at limit_0_15 , dw 0ffffh
        at base_0_15  , dw 0000h
        at base_16_23 , db 00h
        at attribute  , db 93h
        at gran       , db 0Cfh
        at base_24_31 , db 00h
        IEND

code:   ISTRUC  DESCRIPTOR
        at limit_0_15 , dw 0ffffh
        at base_0_15  , dw 0000h
        at base_16_23 , db   00h
        at attribute  , db   9bh
        at gran       , db  0CFh
        at base_24_31 , db   00h
        IEND

Endgdt:

; The stack will share the same Data descriptor. coz of flat model.
Gdt_ip:
                dw      8192                       ; Limit  = FULL
                dd      20000h
Idt_ip:
                dw      0000h
                dd      00000000h

        ;Code area
        ;---------
Start:
        mov     ax , 0800H
        mov     ds , ax
        mov     ax , 5000h
        mov     ss , ax
        mov     sp , 0ffffh

        mov     bx , GDTSEGMENT
        mov     es , bx
        xor     bx , bx
        mov     cx , 32
        mov     si , Gdt

        ;es:bx = 2000:0000
        ;ds:si = 800:gdt

Copygdt:
        mov     al , byte [ds:si]
        mov     byte [es:bx] , al
        inc     si
        inc     bx
        cmp     bx , cx
        jnz     Copygdt


        ; I need to map kernel virtual address from C0000000h
        ; to Phy address 000000h for a length of 1Mb + Kernel size.

        ; Here, we create  identity mapping for 8000h.
        ; Coz, Intel demands such a mapping to exist, for the jump
        ; instruction after enabling paging to succeed. Hence...
        ; This Mapping is needed only during the period of JUMP instruction.
        ; After which this can be discarded..coz we no longer dwell in that
        ; virtual address space.
        ; GDT address should also be mapped. Since GDTR address is a linear
        ; address, we need a mapping for that. Since GDT is copied to some
        ; Other location (2000:0000 => 20000), this linear address should also
        ; have a mapping. Later on GDT val is changed to C0020000 in
        ; main.c.

        mov     bx , KPGD
        mov     es , bx
        xor     bx , bx
clearkpgd:
        mov     [es:bx] , byte 0
        inc     bx
        cmp     bx , 1000h
        jnz     clearkpgd

        mov     bx , 3100h
        mov     es , bx
        xor     bx, bx

clearfirstpage:
        mov     [es:bx] , byte 0
        inc     bx
        cmp     bx ,1000h
        jnz     clearfirstpage

        mov     bx , KPGD
        mov     es , bx
        mov     eax , 00032000H                 ;Address of page table
        or      eax , PG_PRESENT
        or      eax , PG_WRITE
        xor     bx , bx
        mov     [es:bx] , dword EAX             ;Update the 0th entry Page Directory.
        mov     bx , 3200h
        mov     es  , bx
        mov     bx ,  32                        ;8th entry in PG-Table
        mov     eax , 8000h                     ; V r in the 8th page of memory
        or      eax , PG_PRESENT
        or      eax , PG_WRITE
        mov     [es:bx], dword eax

        mov     eax , 20000h                    ;32nd entry. For GDT.
        or      eax , PG_PRESENT
        or      eax , PG_WRITE
        mov     bx , 128
        mov     [es:bx] , dword eax


        ; Here, v map the Kernel virtual addresses starting from C0000000h
        ; Virtual address start =  C0000000h
        ; Mapping length is 1mb + Kernel Text + Static data
        ; Physical address start = 00000000h
        mov     bx , KPGD
        mov     es , bx
        mov     eax , 00031000H         ;Address of Page Table
        or      eax , PG_PRESENT
        or      eax , PG_WRITE
        mov     bx ,  PD_START
        mov     [es:bx], dword eax              ;Update the 768th entry in Page Directory.
        mov     bx , 3100h
        mov     es  , bx
        xor     bx ,  bx
        mov     eax , 000000h

        ;For this mapping,  es:bx is the representative of Virtual address and
        ;EAX is the Physical address.

        cli
Map:
        or      eax , PG_PRESENT
        or      eax , PG_WRITE
        mov     [es:bx] ,eax
        add     bx , 4
        add     eax , 1000h
        cmp     bx ,  0400h  + (KERNEL_PAGES)*4
        jl      Map

        ; Debug display onto video memory
        mov     bx , 0b800h
        mov     es , bx
        xor     bx , bx
        mov     [es:bx], byte 'S'
        mov     [es:bx + 1] , byte 70h
        ; End Debug display

        cli
        push    ds
        mov     ax , 40h
        mov     ds , ax
        ;mov    [ds:067h] , word Secreal
        ;mov    [ds:069h] , word 800h
        pop     ds

        call    switch2pr
        cli
        mov     eax , 30000h
        mov     cr3 , eax
        lgdt    [ds:Gdt_ip]
        lidt    [ds:Idt_ip]
        mov     eax , 80000001h
        mov     cr0 , eax

        db      066h
        db      0eah
        dd      8000h + Tempcode
        dw      0010h

[BITS 32]
Tempcode:
        mov     ax , 8
        mov     ds , ax
        mov     ss , ax
        mov     esp , 0C009efffh
        mov     es , ax
        mov     ss , ax
        mov     gs , ax
        mov     fs , ax
        xor     eax , eax
        xor     ebx , ebx
        xor     ecx , ecx
        xor     edx , edx
        xor     edi , edi
        xor     esi , esi
        xor     ebp , ebp

        db      0eah
        dd      0C0100000h
        dw      0010h



;Control should not reach here. It occurs if  there is an error in
;jumping into kernel. Mainly due to faulty page setup, while enabling
;paging, or missing linear mapping for GDT etc.. Refer intel manual for more
;details.

[BITS 16]

Secreal:
        mov     bx , 0b800h
        mov     es , bx
        mov     bx , 2000h
        mov     [es:bx], byte 'R'
        jmp     Secreal



;----------------------------------------------------------------------------

                switch2pr:
; This procedure gates A20 line , saves 8259 status &
; programs CMOS ports for entering into protected mode.

; IMPORTANT NOTE FOR SRC CODE READERS:
; BIOS area for exit address & MSW & GDT aren't' programmmed here inorder
; to maintain generality.They should be done after | before this call appropriately.
; Issue LMSW after this call. This proc doesn't get u into PM.
;----------------------------------------------------------------------------
                cli
                mov     ah,0dfh
                call    gate_a20   ;gate a20 line
                or      al,al
                jz      Okfine
Tightloop:
                Jmp     Tightloop       ;Get caught in a loop..A20 failed.
                ;A20 GATED ( hip hip hurrah !!! )
Okfine:
                in      al,INTA01        ;save 8259 status in our
                mov     [ds:i82591],al        ;data segment
                in      al,INTA02
                mov     [ds:i82592],al
                mov     al,00fh          ; shut down command index
                out     070h,al          ; cmos port
                jmp     short Dely       ;i/o delay.
Dely:           mov     al,5             ;time delay
                mov     al,5             ;Ask CMOS to flush keyboard and jump
                out     071h,al          ;via "0040:067". See reference on CMOS
                sti
                ret


        gate_a20:
                cli
                call    empty_8042
                jnz     gate_exit
                mov     al,0d1h
                out     064H,al      ;8042 status_port(dual in fn. according to r/w status)
                call    empty_8042
                jnz     gate_exit
                mov     al,ah
                out     060H,al      ;8042 port_a
                call    empty_8042
        gate_exit:
                ret

        empty_8042:
                push    cx
                xor     cx,cx
        E82:
                in      al,064h   ;status port 8042
                and     al,02h
                loopnz  E82
        E83:
                pop     cx
                ret
        times (1024 -($ - $$)) db 0
