; PM Boot loader..
; Author Sarnath. Dec 2nd.
; The boot loader is slow coz of constant switch to PM and real mode.
; I need to revamp this boot loader,  one day.

NUM_TRACKS_TO_LOAD      EQU     2       ; Change this if u want 2 change the
                                        ; number of tracks to load.
                                        ; 1 Track = 36 sectors = 18k
                                        ; 2 Tracks = 36K of data.
SECTORS_TO_LOAD         EQU     3       ; Can be 1 or 3
                                        ; Any other value , will flop.

STRUC   DESCRIPTOR
                limit_0_15:      resw      0001h
                base_0_15:       resw      0001h
                base_16_23:      resb      01h
                attribute:       resb      01h
                gran:            resb      01h
                base_24_31:      resb      01h
ENDSTRUC

BOOTSEG         EQU     07C0H    ; 0000:7c00
BUFFERSEG       EQU     0800H    ; 0800:0000
STACKSG         EQU     9000H    ; 9000:0000
CODE_ACCESS     EQU     9BH
DATA_ACCESS     EQU     93H
BIOS_DATA       EQU     40h

INTA01          EQU     021H    ;8259 port addresses.
INTA02          EQU     0A1H


ABSOLUTE 0x67
                IO_ROM_INIT     resw      1               ; Return address while returning from PM to realmode
                IO_ROM_SEG      resw     1

[BITS 16]

SECTION .text
; This code will b present in the boot area of the floppy.....

        mov     ax , BOOTSEG
        mov     ds , ax

        mov     ax , STACKSG
        mov     ss , ax                         ;set up a stack
        mov     sp , 0FFFFh

        mov     ax , 0600h                      ;clrscr();
        mov     bh , 07h                        ;attribute byte
        xor     cx , cx
        mov     dx , 184fh
        int     10h

        mov     bx , BUFFERSEG
        mov     es , bx
        xor     bx , bx                         ;es:bx = 0800 : 0000
        mov     cl , 04h                        ;From 4th sector.
        mov     ch , 00h
        mov     dl , 00h                        ; Floppy boot
        mov     dh , 00h

Load:
        mov     ah , 02h                        ;Read from floppy
        mov     al , SECTORS_TO_LOAD
        push    dx
        push    cx
        push    es
        push    bx
        int     13h
        cli
        call copyout                         ; Copy out in PM space
        pop     bx
        pop     es
        pop     cx
        pop     dx
        jnc     NoError
             .386
        mov     eax , 200h * SECTORS_TO_LOAD
        sub     [CurrentPtr] , eax
             .8086
        jmp      Load


NoError:
        mov     bx , BUFFERSEG
        mov     es , bx
        xor     bx , bx                         ;Every time load at 0800:0000
        add     cl , SECTORS_TO_LOAD
        cmp     cl , 19                         ; 18 Sectors per side
        jl      Load
        mov     cl , 01h
        inc     dh                              ; Increment head
        and     dh , 01h
        cmp     dh , 00h
        jnz     Load
        inc     ch                              ; 1 Track over. 36 sectors of 512 = 18k = 1 track !
        cmp     ch , NUM_TRACKS_TO_LOAD         ; 18k Loaded.. 1 track completely loaded !!!
        je      JumpIntoKernel
        jmp     Load


JumpIntoKernel:

        mov     bx , 0800h
        mov     es , bx
        xor     bx , bx                         ; 800:000
        mov     ah , 02h                        ; Read
        mov     al , 02h                        ; 2 sectors.
        mov     cl , 02h                        ; Sector
        mov     ch , 00h                        ; Track
        mov     dl , 00h                        ; Floppy boot
        mov     dh , 00h                        ; Head
        int     13h

        mov     dx ,03f2h
        xor     al , al
        out     dx , al

        mov     bx , 0b800h
        mov     es , bx
        xor     bx , bx
        mov     [es:bx] , byte '2'

        db      0eah
        dw      0000h
        dw      0800h                           ;Jump into second stag



CurrentPtr      dd      00100000H   ; Start of O1 mB

copyout:
                mov     ax , BOOTSEG
                mov     ds , ax
                mov     [ds:prevsp] , sp

                call    switch2pr

                push    ds
                mov     ax , BIOS_DATA
                mov     ds , ax
                mov     ax , ReturnToRealmode
                add     ax , 7c00h
                mov     [IO_ROM_INIT] , word ax
                mov     [IO_ROM_SEG]  , word 0000h
                pop     ds

                cli
                lgdt    [ds:LGDT_IP]       ; Don't use cs:fword ptr ... CS = 0x0000 ; not 0x07c0 :-(
                mov     ax , 1
                lmsw    ax                      ;Yes, into PM...

                db      0eah
                dw      7c00h + protected_code  ;Since coz of flat model
                dw      0010h
protected_code:
                cli
                mov     ax , 08h
                mov     ds , ax
                mov     ss , ax
                mov     esp , 0ffffh                    ;Set up a stack at 64k boundary
                cli

                ; Source = 8000h, 512 bytes
                ; Destination  =  CurrentPtr
                xor     ecx , ecx
                mov     esi , 8000h
                mov     edi , CurrentPtr
                add     edi , 7c00h              ;Get the correct address of CurrentPtr
                mov     edi , dword [ds:edi] ; Get the value stored in it.
                cli

Copybytes:
                mov     al , byte [ds:esi]
                mov     [ds: edi] , al
                inc     edi
                inc     esi
                inc     ecx
                cmp     ecx , 200h*SECTORS_TO_LOAD
                jnz     Copybytes

                mov     ebx , CurrentPtr
                add     ebx , 7c00h
                mov     [ds:ebx] , edi        ;Updtate CurrentPtr value.

                   ;Debug display
                mov     ebp , dotcount
                add     ebp , 7c00h
                mov     ebx , dword [ds:ebp]
                mov     [ds:ebx] , byte '.'
                mov     [ds:ebx+1] , byte 07h
                add     [ds:ebp] , dword 2
                   ;end Debug display


                mov     al,0FEH                         ;out 8042 status port. shut_cmd
                out     064H,al
                sti
Loo:
                hlt
                jmp    Loo

                dotcount      dd      0b8000h
                i82591        resb    1
                i82592        resb     1

ReturnToRealmode:
                cli
                mov     ax , BOOTSEG
                mov     ds , ax
                mov     ax , STACKSG
                mov     ss , ax
                mov     sp , word [ds:prevsp]

                mov     ah , 0ddH
                call    gate_a20
                mov     al , byte [ds:i82591]
                out     INTA01 , al
                mov     al , byte [ds:i82592]
                out     INTA02 , al
                ret

;----------------------------------------------------------------------------
                switch2pr:
; This procedure gates A20 line , saves 8259 status &
; programs CMOS ports for entering into protected mode.

; IMPORTANT NOTE FOR SRC CODE READERS:
; BIOS area for exit address & MSW & GDT aren't' programmmed here inorder
; to maintain generality.They should be done after | before this call appropriately.
; Issue LMSW after this call. This proc doesn't get u into PM.
;----------------------------------------------------------------------------
[BITS 16]
                cli
                mov     ah,0dfh
                call    gate_a20   ;gate a20 line
                or      al,al
                jz      Okfine

Tightloop:
                Jmp     Tightloop       ;Get caught in a loop..A20 failed.

                ;A20 GATED ( hip hip hurrah !!! )
Okfine:

                in      al,INTA01        ;save 8259 status in our
                mov     [ds:i82591],al        ;data segment
                in      al,INTA02
                mov     [ds:i82592],al

                mov     al,00fh          ; shut down command index
                out     070h,al          ; cmos port

                jmp     short Dely       ;i/o delay.


Dely:           mov     al,5             ;time delay
                mov     al,5             ;Ask CMOS to flush keyboard and jump
                out     071h,al          ; via "0040:067". See reference about CMOS

                sti
                .8086
                ret


                gate_a20:
                                CLI
                                CALL    EMPTY_8042
                                JNZ     GATE_EXIT
                                MOV     AL,0D1H
                                OUT     064H,AL      ;8042 status_port(dual in fn. according to r/w status)
                                CALL    EMPTY_8042
                                JNZ     GATE_EXIT
                                MOV     AL,AH
                                OUT     060H,AL      ;8042 port_a
                                CALL    EMPTY_8042

                GATE_EXIT:
                                RET

                EMPTY_8042:
                                PUSH    CX

                                XOR     CX,CX

E82:                            IN      AL,064H   ;status port 8042
                                AND     AL,02H
                                LOOPNZ  E82

E83:                            POP     CX
                                RET


GDT:

Null:           ISTRUC DESCRIPTOR
                at limit_0_15, dw 00h
                at base_0_15 , dw 0000h
                at base_16_23, db 0000h
                at attribute , db 00h
                at gran    ,   db 00h
                at base_24_31, db 00h
                IEND

Data:           ISTRUC DESCRIPTOR
                at limit_0_15 , dw 0ffffh
                at base_0_15  , dw 0000h
                at base_16_23 , db 0000h
                at attribute  , db 92h
                at gran       , db 0Cfh
                at base_24_31 , db 00h
                IEND

Code:           ISTRUC DESCRIPTOR               ;16-bit code descr
                at limit_0_15 , dw 0ffffh
                at base_0_15  , dw 0000h
                at base_16_23  , db   00h
                at attribute  , db   9bh
                at gran       , db   08Fh
                at base_24_31 , db   00h
                IEND

Dcode           ISTRUC DESCRIPTOR               ;32-bit code descriptor
                at limit_0_15 , dw 0ffffh
                at base_0_15  , dw  0000h
                at base_16_23 , db    00h
                at attribute  , db    9bh
                at gran       , db   0CFh
                at base_24_31 , db   00h
                IEND

; The stack will share the same Data descriptor. coz of flat model.

LGDT_IP:
                dw      48                       ; Limit
                dd      7C00H + GDT       ; Base of GDT

prevsp          resw            1

                times (510 -$ + $$) db 0
                db 55h
                db 0aah
