/*
 * Reference: ELF specification.
 */

#ifndef __ELF32_H__
#define __ELF32_H__

/* 
 * ELF data types 
 */
#define ELF32_Addr 	void *
#define ELF32_Half 	unsigned short
#define ELF32_Off 	unsigned int
#define ELF32_Sword 	int
#define ELF32_Word 	unsigned long
#define BYTE		unsigned char

/* 
 * e_ident[] indices and valid values 
 */
#define ELF_MAGIC	0x7f /* Zeroth byte of e_ident[] array */

#define EI_CLASS	0x04 /* Fourth byte of e_ident[] array */
#define ELFCLASSNONE	0x00
#define ELFCLASS32	0x01
#define ELFCLASS64	0x02

#define EI_DATA		0x05 /* 5th byte */
#define ELFDATANONE	0x00
#define ELFDATA2LSB	0x01
#define ELFDATA2MSB	0x02

#define EI_VERSION	0x06 /* 6th byte */
#define EI_PAD		0x07 /* 7th byte */

#define ET_NONE		0x0000 /* Something invalid */

/* 
 * #defines for the "e_type" values. 
 */

#define ET_REL		0x0001	/* Relocatable file */
#define ET_EXEC		0x0002	/* Executable file */
#define ET_DYN		0x0003	/* Shared object file */
#define ET_CORE		0x0004	/* Core file */
#define ET_LOPROC	0xff00	/* Processor-specific.*/
#define ET_HIPROC	0xffff	/* Processor-specific. */

/* 
 * #defines for e_machine 
 */
#define EM_M32		0x0001
#define EM_SPARC	0x0002
#define EM_386		0x0003 /* For 386 machines. */
#define EM_68K		0x0004
#define EM_88K		0x0005
#define EM_860		0x0007
#define EM_MIPS		0x0008
#define EM_MIPS_RS4_BE	0x000A


/* 
 * #defines for e_version 
 */
#define EV_NONE		0x00000000
#define EV_CURRENT	0x00000001



/*
 * The ELF Header
 */
#define EI_NIDENT	16
typedef struct {
	BYTE  	   e_ident[ EI_NIDENT];
	ELF32_Half e_type;
	ELF32_Half e_machine;
	ELF32_Word e_version;
	ELF32_Addr e_entry;
	ELF32_Off  e_phoff;
	ELF32_Off  e_shoff;
	ELF32_Word e_flags;
	ELF32_Half e_ehsize;
	ELF32_Half e_phentsize;
	ELF32_Half e_phnum;
	ELF32_Half e_shentsize;
	ELF32_Half e_shnum;
	ELF32_Half e_shstrndx;
}ELFHeader_t;

/* 
 * Section headers and other section related #defines 
 */
typedef struct {
	ELF32_Word	sh_name;
	ELF32_Word	sh_type;
	ELF32_Word	sh_flags;
	ELF32_Addr	sh_addr;
	ELF32_Off	sh_offset;
	ELF32_Word	sh_size;
	ELF32_Word	sh_link;
	ELF32_Word	sh_info;
	ELF32_Word	sh_addralign;
	ELF32_Word	sh_entsize;
} ELF32_Shdr;

/*
 * The following are the various values that
 * can be assumed by the "sh_type" field of
 * the "ELF32_Shdr" structure.
 */
typedef enum sh_type {  SHT_NULL = 0, \
		 	SHT_PROGBITS = 1,\
		 	SHT_SYMTAB = 2,\
		 	SHT_STRTAB = 3,\
		 	SHT_RELA = 4,\
		 	SHT_HASH = 5,\
		 	SHT_DYNAMIC = 6,\
		 	SHT_NOTE = 7,\
		 	SHT_NOBITS = 8,\
		 	SHT_REL = 9,\
		 	SHT_SHLIB = 10,\
		 	SHT_DYNSYM = 11,\
		 	SHT_LOPROC = 0x70000000,\
		 	SHT_HIPROC = 0x7fffffff,\
		 	SHT_LOUSER = 0x80000000,\
		 	SHT_HIUSER = 0xffffffff 
		} sectionType_t;

/*
 * The following are the special section Indices.
 */
#define SHN_UNDEF	0x00
#define SHN_LORESERVE	0xff00
#define SHN_LOPROC	0xff00
#define SHN_HIPROC	0xff1f
#define SHN_ABS		0xfff1
#define SHN_COMMON	0xfff2
#define SHN_HIRESERVE	0xffff

#endif
