/*
 *  ELF learning exercise. 
 *
 */ 


#include <sys/mman.h>
#include <fcntl.h>
#include <stdio.h>
#include "elf32.h"

#define ASSERT(x)	\
		if (!(x) ) {\
			printf("\n Assertion ##x failed");\
			exit(1);\
		}

#define ident(x,i)	x->e_ident[i]
#define MK_P(base, offset) ((char *)base + offset)


ELFHeader_t *elfHeader;
ELF32_Shdr *sectionHeader;

int main(int argc, char *argv[])
{
	int fd,ret,i;

	if ( argc != 2 ) {
		printf("\nUsage: elf32 filename");
		exit(1);
	}

	fd = open( argv[1], O_RDONLY );
	if ( fd == -1 ) {
		printf("\nFile not existing.");
		exit(1);
	}

	ret = (int) mmap((void *)NULL, 0x3000, PROT_READ, MAP_SHARED, fd, 0x00 );
	if ( ret != 0 ) {
		perror("mmap");
	}

	printf("\n Checking for validity ");
	elfHeader = (ELFHeader_t *)ret;
	if ( elfHeader->e_ident[0] != ELF_MAGIC ) {
		printf("\n Not a valid ELF file");
		exit(1);
	}
	ASSERT( ident(elfHeader,1) == 'E');	
	ASSERT( ident(elfHeader,2) == 'L');	
	ASSERT( ident(elfHeader,3) == 'F');	
	printf("\n The given file seems to be an ELF. Proceeding...");
	
	printf("\n Section Offset = %x(in hex)", elfHeader->e_shoff);
	printf("\n Section entry size = %x(in hex)", elfHeader->e_shentsize);
	printf("\n Num section entries = %x(in hex)", elfHeader->e_shnum);
	printf("\n Theoretical size of section entry=%x(in hex)", 
						          sizeof(ELF32_Shdr));
/*
 * Now, peruse the section header table and dump
 * info about various sections in the ELF.
 */
	sectionHeader = ( ELF32_Shdr *) MK_P(ret, elfHeader->e_shoff);
	for (i = 0; i < elfHeader->e_shnum; i++ ) {

		printf("\n Name = %x", sectionHeader[i].sh_name);
		if (sectionHeader[i].sh_type == SHT_STRTAB ) {
			char *s;

			s = (char *)MK_P(ret, sectionHeader[i].sh_offset);	
			printf("\nString = %s",s+0x53);
			sync();
		}
	}
	return 0;
}
