/*
 *              Author :- Sarnath.K.  22-Jan-2001
 *              This file contains general kernel configuration, memory map of
 *              different kernel objects etc..
 */

#ifndef __CONFIG_H__
#define __CONFIG_H__

#include <i386/page.h>

/*
 * Kernel Configuration
 */

#define KERNPAGES       9
/*
 * The same as in 2ndstage.asm in boot directory.
 * If this is changed, the one in boot directory should also be
 * changed.
 */

#define KERNEL          0xC0000000 /* Virtual address -> start of kernel */
#define KERNEL_CS       0x10    // Same as in boot/2ndstage.asm
#define KERNEL_DS       0x08    // Same as in boot/2ndstage.asm
                                // IF this is changed, change the
                                // ENTER_EXCEPTION macro in kvm.c
#define KERNEL_TSS      0x18
/*
 * KERNEL_TSS is used only for the first Task Switch.
 * See main.c
 */

#define THREAD_TSS      0x20
/*
 * Explicitly created user or kernel threads shall use THREAD_TSS
 * as their TSS value. All of them share the same TSS value.
 */
#define LOADPOINT       0x100000 /* physical addresses */
#define DMABUFFER       0x10000
#define GDTBASE         0x20000
#define GDTLIMIT        0xffff
#define KPGD            0x30000
#define FIRSTPTABLE     0x31000
#define IDTBASE         0x32000
#define IDTLIMIT        0x800
#define FREEPTABLE      0x33000
#define MAPTABLE        KERNEL + FREEPTABLE
#define FREE_VIRT       KERNEL + 0x34000
#define EARLY_PTABLE    0x35000
#define FREEMEM         0x40000

/*
 * Accounting for  1 GB virtual address space
 * Refer -> file kvm.c in "kernel directory, and (mm/vm.h)
 */
#define VA_MAX                  (~(0))
#define PA_MAX                  (~(0))
#define STATIC_KERNEL           ( KERNEL + LOADPOINT )
#define STATIC_KERNEL_LIMIT     ( KERNPAGES * PAGE )
#define KERNEL_STACK            ( 0x90000 )     
/*  
 * The same as in boot/2ndstage.asm. This stack
 * is used during the initial stages. After this
 * this area is ignored. Each thread has its own
 * kernel stack.
 */

#define KERNEL_STACK_TOP        ( 0x9efff )
/*
 * This is for defining the video buffer segment.
 */
#define START_VIDEOBUFFERS      ( 0x9F000 )
/*
 * Actually from 0xA0000 . But Linux source says that IBM has messed up sthg
 * in 9f000h. Hence me too following the same.
 * I m not sure about the above statement. No denigration intended.
 */
#define END_1MB                 ( 0xFFFFF )
#define VIDEOLIMIT              ( END_1MB - START_VIDEOBUFFERS + 1)

/*
 * For "KmemSeg".
 */
#define KMEM_SEG_BASE           ( 0xE0000000 )
#define KMEM_LIMIT              ( 0x10000000 )


/*
 *      Accounting  for initial physical memory, populated by kernel
 */
#define START_LOWMEM            ( 0x0 )
#define END_LOWMEM              ( FREEMEM )
#define START_KERNELPAGES       ( LOADPOINT )
#define END_KERNELPAGES         ( LOADPOINT + KERNPAGES * PAGE )
#endif
