/*
 *      Data structures regarding segment descriptors.
 *      -------------------------------------------
 *                                                      5thMay2001.
 *                                                      Sarnath.K
 */

#ifndef __DESCR_H__
#define __DESCR_H__

#include <generic/types.h>
/*
 *  There are two types of descriptors:
 *  1. System Descriptor
 *  2. Non-System Descriptor.
 *  A Non-System descriptor doesn't mean that the descriptor is owned
 *  by an appln program.
 *  The DPL field of a descriptor determines the privilege level of a
 *  descriptor.
 *  Not all descriptors with privilege level 0 are system descriptors.
 *  The 4th bit in the 6th byte of a descriptor tells u what type of
 *  descriptor this is. If it 0, then it is a system segment descriptor
 *  else it is a non-system descriptor.
 *                                                      -- Sarnath.K
 *  The TYPE field + SYSTEM field => Tells the processor, what type
 *  of descriptor it is.
 */

typedef struct  segment_descriptor {
        ushort  limit_0_15;
        ushort  base_0_15;
        byte    base_16_23;
        byte    type:4;
        byte    system:1;
        byte    dpl:2;
        byte    present:1;
        byte    limit_16_19:4;
        byte    avail:1;
        byte    zero:1;
        byte    operand_size:1;
        byte    granualarity:1;
        byte    base_24_31;
}seg_descr_t;

typedef struct ldt_flat_entries {
        seg_descr_t code;
        seg_descr_t data;
/* stack uses the same segment as data */
}ldt_t;

#define NONSYSTEM_SEGMENT       1
#define SYSTEM_SEGMENT          0
#define PAGE_GRANUALAR          1
#define BYTE_GRANUALAR          0
#define BIG                     1
#define SMALL                   0
#define KPL                     0
#define APL                     3
#define THIRTYTWO               1       // Default Operand Size in case of CodeSeg
#define SIXTEEN                 0
/*
 * Various #defines for the TYPE field of the descriptor.
 * These are all Types for NON-SYSTEM descriptors.
 * Directly from Intel Manual.
 */
        /* This is for Data segments */
#define RDONLY                          (0x00)
#define RDONLY_ACCESSED                 (0x01)
#define RDWR                            (0x02)
#define RDWR_ACCESSED                   (0x03)
#define RDONLY_EXPAND_DOWN              (0x04)
#define RDONLY_EXPAND_DOWN_ACCESSED     (0x05)
#define RDWR_EXPAND_DOWN                (0x06)
#define RDWR_EXPAND_DOWN_ACCESSED       (0x07)
        /* This is for code segments */
#define EXECONLY                        (0x08)
#define EXECONLY_ACCESSED               (0x09)
#define EXEC_READ                       (0x0A)
#define EXEC_READ_ACCESSED              (0x0B)
#define EXECONLY_CONFORMING             (0x0C)
#define EXECONLY_CONFORMING_ACCESSED    (0x0D)
#define EXEC_READ_CONFORMING            (0x0E)
#define EXEC_READ_CONFORMING_ACCESSED   (0x0F)

/*
 *      Below are #defines from the INTEL manual for the TYPE
 * field of SYSTEM descriptors.
 */
#define AVAIL_TSS_16BIT         (0x01)
#define LDT                     (0x02)
#define BUSY_TSS_16BIT          (0x03)
#define CALL_GATE_16BIT         (0x04)
#define TASK_GATE               (0x05)
#define INTERRUPT_GATE_16BIT    (0x06)
#define TRAP_GATE_16BIT         (0x07)
#define AVAIL_TSS_32BIT         (0x09)
#define BUSY_TSS_32BIT          (0x0B)
#define CALL_GATE_32BIT         (0x0C)
#define INTERRUPT_GATE_32BIT    (0x0E)
#define TRAP_GATE_32BIT         (0x0F)

#endif
