/*
 *      File holds the 386+ specific task-data-structures.
 *                                                      -- Sarnath.K
 */
#ifndef __TSS_H__
#define __TSS_H__

#include <generic/types.h>

#define BUSY_TSS (0x0b)         // The BusyBit is set in this case.
#define LAZY_TSS (0x09)         // The BusyBit is not set in this case.
/*
 * The TSS descriptor.
 */
struct tss_descr {
        ulong   limit_0_15:16;
        ulong   base_0_15:16;
        byte    base_16_23;
        byte    one:1;
        byte    busy:1;
        byte    zero:1;
        byte    nextone:1;
        byte    nextzero:1;
        byte    dpl:2;
        byte    present:1;
        byte    limit_16_19:4;
        byte    avail:1;
        byte    alwayszero:2;
        byte    granualarity:1;
        byte    base_24_31;
};

/*
 * The Task State Segment.
 */
typedef struct tss {
        ushort back_link;
        ushort unused0;
        ulong  esp0;
        ushort ss0;
        ushort unused1;
        ulong  esp1;
        ushort ss1;
        ushort unused2;
        ulong  esp2;
        ushort ss2;
        ushort unused3;
        ulong  cr3;
        ulong  eip;
        ulong  eflags;
        ulong  eax,ecx,edx,ebx,esp,ebp,esi,edi;
        ushort es;
        ushort unused4;
        ushort cs;
        ushort unused5;
        ushort ss;
        ushort unused6;
        ushort ds;
        ushort unused7;
        ushort fs;
        ushort unused8;
        ushort gs;
        ushort unused9;
        ushort ldt_selector;
        ushort unused10;
        ushort  Tbit:1;
        ushort  unusedbits:15;
        ushort  IOmap_base;
} tss_t;
/* All unused bits & bytes in TSS should be zeroes */
#endif
