/*
 * File created on  May-4-2001.
 * This file shall contain all information about a process.
 *                      - Sarnath.K
 */
#ifndef __THREAD_H__
#define __THREAD_H__

#include <generic/types.h>
#include <mm/vm.h>
#include <i386/tss.h>
#include <i386/descr.h>

/*
 *                      Big comments :)
 *              If Processes have a flat address space
 *              -------------------------------------
 * PROS:
 *      If i m not using a segmented approach, the number of segments per
 * process would be two; one for DS and other for CS. If i have a
 * LDT (8192 entries) per-process out of which i'm going to use only
 * 2 entries, allocation of a LDT would be a waste of memory. An LDT is
 * 64kbytes big. LDT is mainly meant for segmentation of an address-space.
 * Still we will have segments in s/w level. Although processor sees a single
 * flat segment, the kernel partitions it into segments.
 *      Thanx to INTEL. Every time we go for LDT, they need not be 64K long.
 * Rather we can specify the LIMIT of the LDT in the "LLDT" instruction.
 * Having a "small" LDT for each process would simplify the task.
 *
 * OTHER CONSIDERATIONS:
 * 1.   If we have all process's selectors in GDT itself,
 * the isolation of processes gets reduced. Process A can easily load a segment
 * register corresponding to Process B and write into Process B's Virtual
 * address space. ( without any access problems, since both A and B's privilege
 * levels are same = 3 ). However since Paging Unit is active, process A
 * cannot spoil Process B's physical image. In fact process A would be
 * destroying it's own code and data. Since Process A is now seeing it's own
 * address space with some other protection information, results would be bad
 * This is not a big problem. Process A should suffer if he is behaving
 * abnormally. But as the supervisor, we wont be able to catch this problem.
 * Process A will go mad and we can just watch him go mad unless he gets
 * caught via an illegal page-fault or any other.  BUT THIS PROBLEM CAN BE
 * SOLVED BY HAVING A SEPERATE LDT FOR EACH PROCESS.
 *
 * 2.   There is yet another problem if we have no LDTs. Since Each process
 * owns 4 entries in the GDT ( One for TSS, One for DS, One for CS , One for LDT),
 * the total number of processes that can be supported on the system would be
 * limited. For exp, if each process owns 4 entries in GDT
 * then total number of processes that can be supported in the
 * system = ( 8192-3 )/4 = 8189/4 > 2000 simultaneous processes
 * Processes can free and allocate GDT entries each time they are contex
 * switched in and out, thereby making judicious use of GDT table.
 *
 * 3.   We cannot enforce minute protection information in an address space.
 * coz we have only two segments.But protection can be enforced in Paging.
 * by marking the physical page as read-only or anything appropriate.
 *
 *      The effects on the memory allocator in a user-address space.
 *      -----------------------------------------------------------
 *      In case of a segmented address space, the memory allocator can either
 * return a "near address" or "far address". In case of a far memory allocation
 * we can specify the protection also if we need. Creating a new segment for
 * a process would mean creating a new entry in LDT. Segments may overlap.
 * One for Code, One for uninitialized Data ( DS shall defaultly pointing
 * towards this segment ), Dynamic allocations form a seperate segment, Stack
 * forms a seperate segment etc..
 *      Whereas in case of flat model, there is no concept of near or far.
 * All address belong to the same segment. Quite simpler management !
 *
 *              ALL ABOVE ARE JUST CONSIDERATIONS. IMPLEMENTATIONS MIGHT
 * BE DIFFERENT. BUT I HAVE DOCUMENTED IMPLEMENATIONS AT APPROPRIATE PLACES.
 *
 *                                                              --Sarnath.K
 */

/*
 *                      PROCESS ADDRESS SPACE
 *                      ---------------------
 *      This is what i have decided to implement.
 *      1. I have preferred FLAT model for a process's address space.
 * Processor sees only 2 segments (Code and Data (Each 0 - 4G)) for a process.
 * The kernel takes care of segments within process address space.
 *      2. Each Process has it's own LDT. This is mainly for protection and
 * isolation of tasks virtually from one another.
 *      3. Each process own a TSS, LDT in the GDT.
 *      4. Size of LDT = 0x10 for all processes.
 *      5. I'll allocate 16 bytes for LDT. Because i very well know that
 * only 2 segments are used in it. The limit of LDT is small in this case.
 * Hence i would set the LIMIT of LDTR to 16. This way, if the process trys
 * to load a segment register with an illegal value, he will be killed.
 * I allocate all LDTs at boot time. ( Note LDT address is Linear address.)
 *      6. Flat model gives me lot of flexibility.
 *      7. Max number of simutaneous tasks on the system = (8192 - 3)/2 > 4000
 *                                                              --Sarnath.K
 *      There is no restrction on number of tasks. One entry in TSS would
 *      suffice for all tasks. During Each task switch, i would make this TSS
 *      point to the TSS of the corresponding tasks. Thats it. Nothing more
 *      that. This involves tricky multi-tasking. See kernel/thread.c. "switch_to_sytem_thread" 
 */


/*
 * Let me call a process as "THREAD OF EXECUTION".
 * I need to clarify certain things at this stage.
 * UserLevel multi-threading can be managed at user-level itself.
 * A thread library at the user level can be implemented.
 * What support the kernel should extend to this lib is a qn ?
 * Probably, i can implement this using "signal"s as in linux.
 *
 * I feel that giving kernel-level "user-thread" support to processes are
 * unnecessary. A thread at the kernel level can represent a KERNEL-THREAD (or)
 * USER-PROCESS. ( Read first line of comment once again, if you have any
 * discrepancy.)
 * Basically, Threads can be of two types. One is a System Thread and
 * Other is a user thread. They are maintained in seperate run queues.
 * Multiple kernel threads means Preemptive kernel. I should look into these
 * issues.
 *                                                              --Sarnath.K
 */

#define TEXT_START      0x1000
/*
 * This is where in the Process AS, the initial text starts.
 * This helps trapping NULL ptr deference.
 */
#define KERNEL_STACK_SIZE       2*PAGE
/*
 *      stack size for user processes.
 */

#define NR_THREADS      1024
/*
 * Lets have a limitation. Albeit its not really
 * necessary.
 */
#define NR_TICKS        5
enum THREAD_TYPE { SYSTEM, USER, IDLE};


typedef struct sys_thread {
        as_t *tas;              // "tas = kas"
        struct tss *tssp;       // Holds the context.
        struct sys_thread *next_run, *prev_run; // run-queue-of-threads.
        ulong tq;               // Remaining time quantum.
} sys_thread_t;

typedef struct processInfo {
        ulong textLength;
        ulong dataLength;
} processInfo_t;

/*
 * An user_thread represents a user process.
 */
typedef struct user_thread {
        as_t *uas;
        int id;
        struct tss *tssp;
        struct user_thread *next_run, *prev_run; /* Llist of RunQ */
        struct user_thread *next, *prev;        /* Llist of task */
        struct processInfo  *info;
        ldt_t  processLdt;
/*
 *      i386 specific Ldt descriptors for describing segments in the addrSpace.
 *      These segments are trivial and describe 0-3g of entire userAddressSpace.
 *      The real segment management is done in software. i386 features are not
 *      fully utilized. This offers more flexibility in design and
 *      dataStructures. ( More portable ! )
 */
        char    *kernelStack;
        ulong tq;
} user_thread_t;

extern sys_thread_t *sys_run_q; // A circular doubly linked list.
extern user_thread_t *user_run_q; // A circular doubly linked list.
extern sys_thread_t *current_sys_thread; // Currently running thread
extern user_thread_t *current_user_thread; // Currently running thread.

#endif
