/*
 * Author : K.Sarnath. 14-Jan-2001
 * ----- 
 * Description:
 * -----------
 * 	Clock Handler and CPU exception handlers are here.
 * 	This file is the equivalent of linux's "entry.S".
 *
 * Facts:
 * -----
 * Exception types: (Courtesy: Write ur own OS. Web-site)
 *
 * Fault - The return address points to the instruction that executed the
 *         instruction . Problem can b fixed and restarted.
 *
 * Trap -  ret address points to instrn after the one which has completed.
 *
 * Abort - The return address is not always reliably supplied. A program that
 * aborted is never meant to be continued.
 *
 * All Trap Entry Points, still have the FramePointer codings.. ( i.e. 
 * pushl %ebp, mov %esp, %ebp ). I m not using any -fomit-frame-pointer stuff
 * in gcc. This might help debugging at some point of time.
 *
 */

#include <i386/idt.h>
#include <generic/types.h>
#include <config.h>
#include <i386/page.h>
#include <threads/thread.h>

extern pte_t *kpgd;
extern as_t  *cas;
extern register_idt_entry();
extern kernel_pagefault_handler();
extern get_temp_mapping ( ulong );
extern int start_debug;
extern enum THREAD_TYPE current_type;

unsigned long *framePointer = (unsigned long *)NULL;

#define NUM_PROCESSOR_EXCEPTIONS 32


/*
 * Change these macros appropriately, if compiled with -fomit-frame-pointer.
 */

#define ENTER_INTERRUPT asm("pushal");\
                        asm("push %ds");\
                        asm("push %es");\
                        asm("push %fs");\
                        asm("push %gs");\
                        asm("movw %0, %%ax"::"i" (KERNEL_DS));\
                        asm("movw %ax, %ds");\
                        asm("movw %ax, %es");\
                        asm("movw %ax , %fs");\
                        asm("movw %ax, %gs");\
                        asm("movl %%esp, %0"::"m" (framePointer))

#define RETURN_FROM_INTERRUPT asm("pop %gs");\
                              asm("pop %fs");\
                              asm("pop %es");\
                              asm("pop %ds");\
                              asm("popal");\
                              asm ("movl %ebp , %esp");\
                              asm ("popl %ebp");\
                              asm ("iretl")

#define RETURN_WITH_ERRORCODE 	asm("pop %gs");\
                              	asm("pop %fs");\
                              	asm("pop %es");\
                              	asm("pop %ds");\
                              	asm("popal");\
                              	asm ("movl %ebp , %esp");\
                              	asm ("popl %ebp");\
			      				asm("\naddl $0x4, %esp");\
                              	asm ("iretl")


struct exception_table {
        unsigned char exception_num;
        int (*handler)();
};

unsigned long jiffies = 0; /* Time since boot up */
ulong fault_va , errorcode;
short int sel;
extern void delay();

        /* int $0x20 */
int clock_handler()
{
        ENTER_INTERRUPT;
        jiffies ++;
        asm ("movb $0x20 , %al");
        asm ("movw $0x20 , %dx");
        asm ("outb (%dx)");     /* Send EOI */

/*
 * Calls to schedule should be after issuing the EOI.
 */

        switch ( current_type ) {
                case IDLE:
                                schedule();
                                break;

                case SYSTEM:
                                current_sys_thread->tq--;
                                if (!current_sys_thread->tq)
                                        schedule();
                                break;
                case USER:
                                current_user_thread->tq--;
                                if (!current_user_thread->tq)
                                        schedule();
                                break;
        }
        RETURN_FROM_INTERRUPT;
}

        /* int $0x00. fault */
int  divide_exception()
{
        printk (" \nDivide exception ");
        printk (" Reboot computer manually \n");
        while (1);

}

        /* int $0x01. trap or fault */
int debug_exception()
{
	ENTER_INTERRUPT;
        printk ("\n Debug exception caught. \n");
        printk ("Hope at some point of time, this would help \n");
        while(1);

}

        /* int $0x02. trap */
int nmi_exception()
{
	ENTER_INTERRUPT;
        printk("\n NMI exception caught. Reboot manually");
        while (1);
}

        /* int $0x03. trap */
int breakpoint_exception()
{
	ENTER_INTERRUPT;
        printk("\n Breakpoint exception caught. reboot manually \n");
        while (1);
}

        /* int $0x04. trap */
int overflow_exception()
{
	ENTER_INTERRUPT;
        printk("\n Overflow exception caught. reboot manually \n");
        while (1);
}

        /* int $0x05. trap */
int bound_exception()
{
	ENTER_INTERRUPT;
        printk("\n Bound exception caught. reboot manually \n");
        while (1);
}

        /* int $0x06. trap */
int invalidopcode_exception()
{
	ENTER_INTERRUPT;
        printk("\n Invalid opcode  exception caught. reboot manually \n");
        while (1);
}

        /* int $0x07. trap */
int fpunotavailable_exception()
{
	ENTER_INTERRUPT;
        printk("\n FPU not available exveption caught. Reboot manually \n");
        while (1);
}

        /* int $0x08. abort */
int doublefault_exception()
{
	ENTER_INTERRUPT;
        printk("\n Double fault exception caught. Reboot manually \n");
        while (1);
}

        /* int $0x09. abort */
int coprocessorsegmentoverrun_exception()
{
	ENTER_INTERRUPT;
        printk("\n Coprocessor segment overrun  exception caught. Reboot manually \n");
        while (1);
}

        /* int $0x0a. fault */
int invalidtss_exception()
{
	ENTER_INTERRUPT;
        printk("\n TSS is invalid. Reboot manually \n");
        while (1);
}

        /* int 0x0b. fault */
int segmentnotpresent_exception()
{
	ENTER_INTERRUPT;
        printk("\n Segment not present exception caught. Reboot manually \n");
        while(1);
}

        /* int 0x0c. fault */
int stack_exception()
{
	ENTER_INTERRUPT;
        printk("\n Stack exception exception caught. Reboot manually \n");
        while(1);
}

        /* int $0x0d, fault or trap */
int generalprotection_exception()
{
	ENTER_INTERRUPT;
        printk("\n GP exception caught.");
        printk ("\n\n Recent page fault add %x, errorcode %x", fault_va, errorcode);
        panic();
}

        /* int $0x0e, fault */
int pagefault_exception()
{
	ENTER_INTERRUPT;
    asm ("movl %cr2 , %eax");
    asm ("movl %eax , fault_va");
    asm ("movl 0x4(%ebp), %eax");
    asm ("movl %eax , errorcode");

#ifdef __PAGEFAULTDEBUG__
        printk ("\n Got a Pfault at address %x Errorcode = %x" , fault_va ,errorcode);
#endif
        if (fault_va >= KERNEL) {
#ifdef __PAGEFAULTDEBUG__
                printk ("\n OOps ! A kernel fault..Invoking handler \n");
#endif
                kernel_pagefault_handler( fault_va );
        }
        else {
                printk("\n UserPageFault address %x", fault_va);
                printk("\n ErrorCode = %x", errorcode);
                printk("\n cas = %x", cas);
                printk("\n pgdir = %x", cas->pgdir);
                asm("cli");
                userPageFaultHandler( fault_va );
        }
#ifdef __PAGEFAULTDEBUG__
        {
                ulong ptable , pt_offset;
                pte_t *va;
                ulong i;

                printk ("\n \n \n The contents of Kernel Page directory \n");
                ptable = kpgd [ PGDIR_OFFSET (fault_va) ].pte;
                PAGE_ALIGN (ptable);
                printk ("\n The Page Table address = %x", ptable );
                va = (pte_t*)get_temp_mapping ( ptable );
                pt_offset = PGTABLE_OFFSET ( fault_va );
                printk ("\n Phy address mapped for %x = %x", fault_va, va[pt_offset].pte);
        }
#endif
	RETURN_WITH_ERRORCODE;
        asm ("addl $0x4 , %esp ");      /* POP the error code */
        asm ("iretl");
}

        /* int $0x10, fault */
int floatingpointerror_exception()
{
	ENTER_INTERRUPT;
        printk("\n Floating point exception caught. reboot manually \n");
        while (1);
}

        /* int $0x11. fault */
int alignmentcheck_exception()
{
	ENTER_INTERRUPT;
        printk("\n Alignment check exception caught. reboot manually \n");
        while (1);
}

        /* int $0x12, abort */
int machinecheck_exception()
{
	ENTER_INTERRUPT;
        printk("\n machine check exception caught. Reboot manually :-) \n");
        while (1);
}

        /* Reserved ones, int $0x0f, int 19 to 31 */
int reserved_exception()
{
	ENTER_INTERRUPT;
        printk("\n Reserved exception,  so funny . Reboot manually \n");
        while (1);
}


const struct exception_table exceptions[NUM_PROCESSOR_EXCEPTIONS] = {
 { 0, &divide_exception},
 { 1, &debug_exception},
 { 2, &nmi_exception},
 { 3, &breakpoint_exception},
 { 4, &overflow_exception},
 { 5, &bound_exception},
 { 6, &invalidopcode_exception},
 { 7, &fpunotavailable_exception},
 { 8, &doublefault_exception},
 { 9, &coprocessorsegmentoverrun_exception},
 { 10, &invalidtss_exception},
 { 11, &segmentnotpresent_exception},
 { 12, &stack_exception },
 { 13, &generalprotection_exception},
 { 14, &pagefault_exception},
 { 15, &reserved_exception},
 { 16, &floatingpointerror_exception},
 { 17, &alignmentcheck_exception},
 { 18, &machinecheck_exception},
 { 19, &reserved_exception},
 { 20, &reserved_exception},
 { 21, &reserved_exception},
 { 22, &reserved_exception},
 { 23, &reserved_exception},
 { 24, &reserved_exception},
 { 25, &reserved_exception},
 { 26, &reserved_exception},
 { 27, &reserved_exception},
 { 28, &reserved_exception},
 { 29, &reserved_exception},
 { 30, &reserved_exception},
 { 31, &reserved_exception}};

int install_exception_handlers()
{
        int i;

        asm("movw %cs, sel");

        for ( i=0; i<32; i++)
        {
                register_idt_entry(exceptions[i].exception_num, sel, (ulong) exceptions[i].handler, (uchar) 0, (uchar )1, TRAP_GATE);
        }
}
