/*
 *      "main" get control from "2ndstage.asm". This code will be in 1st Mb of
 *      RAM after bootup.
 *      Author -> Sarnath.k.
 *      Jan 1st 2001
 */

#include <i386/idt.h>
#include <i386/descr.h>
#include <i386/tss.h>
#include <i386/page.h>
#include <config.h>
#include <mm/pmem.h>
#include <mm/kmem.h>
#include <mm/vm.h>

#define GDTR     ( KERNEL + GDTBASE )
#define IDTR     ( KERNEL + IDTBASE )

extern printk();
extern PrintkInit(void);
extern initialize_8259();
extern clock_handler();
extern int install_exception_handlers();
extern void memsetup();
extern print_kernel_segments();
extern kmem_init();
extern ulong freemem;
extern int early_stage , biosmem;
extern seg_descr_t *gdt;
extern void idle_kernel_thread();
extern void create_kernel_thread();
extern ulong idle_thread;
int print_kernel_thread();
int another_thread();

char *initProcess = "adamNeve";
as_t *cas;      // Current address space.
pte_t *map_page_table = (pte_t *) FREEPTABLE; /* Phy address */
pte_t *kpgd = (pte_t *) (KERNEL + KPGD);
struct tss *kernel_tss = (struct tss * )NULL;

unsigned short int cs_selector;
unsigned short int ldt_selector;

int start_debug;
#ifdef __XLATDEBUG__
void *array[200];
#endif


main()
{
        ulong i;

/*
 *      See the docs/kernmap file to know "which" objects are "where" in memory
 */
        asm("cli");
        cas = &kas;
        flgdt(GDTR, GDTLIMIT);
        flidt(IDTR , IDTLIMIT);
        asm("push %ds \n pop %ds \n push %es \n pop %es");
        asm("movw %cs , cs_selector");
/*
 *      Do all initialization stuff here.
 */
        early_stage = 1;        // Page-Fault hdlrs behave differently in early_stage.
        PrintkInit();
        initialize_8259();
        install_exception_handlers();
        register_idt_entry(0x20, cs_selector, &clock_handler, 0 ,1, INTERRUPT_GATE);
/*
 *      End of initialization.
 *      8259 is done. Printks are done.
 *      Exception handlers are installed.
 */
        printk ("\n\t\tPhoenix loaded\n ");
        printk ("\t\t===============\n");

        kseg_init();    // This should be the first.
        memsetup();     // This guy makes early_stage = 0; First Page Fault in
                        // the system occurs in this function.
        printk("\n Mem setup is over...");
        kmem_init();
        gdt_init();     // Should be after kmem init.
        printk ("\n kmeminit done..");
        print_kernel_segments();
        i = 0;

#ifdef __PMEMDEBUG__
        printk ("\n PTABLE list => \n");
        print_ptable_list();
#endif

#ifdef __XLATDEBUG__
/*
 *      NOTE: These codes, under "debug", are used to test a particular sub-system.
 *      Source code readers need not worry about this.
 */
        printk ("\n freemem = %x", freemem );
        for ( i=0; i<100; i++)
                array[i] = (void *)kmem_alloc(8190);
        for ( i=0; i<100; i++)
                kmem_free(array[i],8190);

        printk ("\n Done with allocns and freeings..starting a new phase");
        printk ("\n freemem = %x", freemem );
#endif

#ifdef __GDTDEBUG__
        {
                int sel;

                sel = gdt_selector_alloc();
                printk ("Selector = %x \n", sel);
                gdt_selector_free (sel);
                printk("\n Freed..");
                sel = gdt_selector_alloc();
                printk ("Allocated again...Selector = %x \n", sel);
        }
#endif

        ldt_selector = gdt_selector_alloc();
        printk("\n LDT selector = %x", ldt_selector);
/*
 * All applications shall make use of a single LDT selector.
 * During context switch, the LDT selector shall point to
 * the LDT of the correspoding application.
 *                                              Aug/19/2001.
 */
        kernel_tss = (struct tss *) kmem_alloc (sizeof(struct tss) );
        makeZero ( kernel_tss, sizeof(struct tss));
        create_descriptor (&gdt[ (KERNEL_TSS >> 3)],(ulong)kernel_tss,sizeof(struct tss),SYSTEM_SEGMENT, KPL,AVAIL_TSS_32BIT,0,0,PAGE_GRANUALAR);
        asm ("movw $0x18, %ax; ltr %ax"); // KERNEL_TSS = $0x18

        asm("cli");
        sysCreateProcess("adamNeve");
        printk("\n Enabling interrupts...");
        asm("\nsti");
        while (1);
}

int
print_kernel_thread()
{
        while (1) {
                printk ("\n  1. printk");
                delay();
                printk ("\n  2. printk");
                delay();
                printk("\n   3. printk");
                delay();
                delay();
                printk("\n   4. printk");
        }
}

int
another_thread()
{
        while (1) {
                printk("\n 1 chummy");
                delay();
                printk("\n 2 chummy");
                delay();
                printk("\n 3 chummy");
                delay();
                printk("\n 4 chummy");
                delay();
                printk("\n 5 chummy");
                delay();
        }
}

int
delay()
{
        unsigned int i;

        for (i=0;i<0x1fffff; i++);
        return 0;
}

int flgdt(void *base, unsigned int limit) {
   unsigned int i[2];

   i[0] = limit << 16;
   i[1] = (unsigned int) base;
   asm ("lgdt (%0)": :"p" (((char *) i)+2));
   return 0;
}

int flidt(void *base, unsigned int limit) {
   unsigned int i[2];

   i[0] = limit << 16;
   i[1] = (unsigned int) base;
   asm ("lidt (%0)": :"p" (((char *) i)+2));
   return 0;
}

#ifdef __PMEMDEBUG__
int
pmemtest()
{
        ulong pa=0, delaylock = 0;

        do {
                pa = pmem_alloc(SYSTEM);
                printk ("%x ",pa);
                if ( !delaylock && (pa >= 0x100000) ) {
                        delaylock =1;
                        delay();
                }
        }while ( pa != 0xffffffff );
        printk ("\n Breaker= %x" , pa);
        return pa;
}

int
print_ptable_list()
{
        pmem_t *tr = pagetable_head;

        while ( tr != NULL ) {
                printk ("\n PageTable at physical address => %x", ((tr - mem_table) << PAGE_SHIFT));
                tr = tr->link.next_page_table;
        }

}
#endif


