/*
 *  This file is supposed to have the xlation related data structures.
 *                                              -- Sarnath.K
 */

#ifndef __VTOP_H__
#define __VTOP_H__
#include <config.h>
#include <generic/types.h>
#include <i386/page.h>
#include <mm/vm.h>

/*
 * The structure "vtop" helps in identifying the various owners of a pagetable.
 * Inorder to know about the owners of a page-table, get on  to its "pmem_t"
 * structure, and follow the linked list of "vtop"s to find out, what all
 * {address-space, PDE-num} use the page-table.
 * Note:
 *      Regarding memory allocation of vtop structures, i dont use
 * kmem_allocs, coz the problem becomes recursive. All assosiated addresses
 * are physical.
 *                                                      - Sarnath.K
 */

typedef struct vtop {
        ulong pde;
        as_t  *owner;           // Owning Address Space.
        ulong refcnt;           // Number of mappings that thrive on this p-table
        struct vtop *next;      // doubly linked list, physical addresses
        struct vtop *prev;      // Get a temp mapping before indirecting these PTRs.
                                // else u might land up in a panic.
                                //              (or)
                                // i can have an alternate PGDIR in which
                                // address 0 to 3g are mapped linearly and
                                // 3G to 4G should b IIIr to kernel page
                                // directory. In this case, make sure while
                                // allocating phy memory for a vtop structure,
                                // it should be less than 3G physical else i
                                // cant map them in the new page-directory. If
                                // i do so, i'll lose kernel mappings since
                                // kernel resides above 3gb offset. This method
                                // is very suave. The previous method
                                // will just slow down performance due to
                                // constant TLB flushes. but this method is
                                // memory expensive, coz u need 768
                                // more page tables. which = 3mb of RAM.
                                //              Bap re !
                                // I have chosen the Former method. The second
                                // method can also be implemented in a
                                // lazy way of page-table allocation. but let
                                // me play it safer. I'd adopt the 2nd method
                                // some other day if i feel this is a
                                // bottleneck.
}vtop_t;

// I exploit the fact that the structures "ptov_t" and "vtop_t" are same in
// size. So that i can use the routines "get_a_ptov" and "free_a_ptov" for
// allocating memory for these "vtop" structures also. Be sure to pad this
// structure in case the structure "ptov" is changed.
#endif
